////////////////////////////////////////////////////////////////////////////////////////////////////
// Experiment 13 - Default environment map vertex shader
////////////////////////////////////////////////////////////////////////////////////////////////////

////////////////////////////////////////////////////////////////////////////////////////////////////
// ATTRIBUTE
attribute vec3    pos_attr;
attribute vec3    normal_attr;
attribute vec2    uv_attr;	

////////////////////////////////////////////////////////////////////////////////////////////////////
// UNIFORM
uniform mat4	g_mWorldTM;
uniform mat4	g_mViewProjTM;
uniform vec4	  g_vSHAr;
uniform vec4	  g_vSHAg;
uniform vec4	  g_vSHAb;
uniform vec4	  g_vSHBr;
uniform vec4	  g_vSHBg;
uniform vec4	  g_vSHBb;
uniform vec4	  g_vSHC;
uniform float	  g_fSHScaler;

////////////////////////////////////////////////////////////////////////////////////////////////////
// VARYING
varying vec2	  vTexCoord;
varying vec3	  vColor;
varying vec3	  vWorldPos;
varying vec3	  vWorldNormal;
////////////////////////////////////////////////////////////////////////////////////////////////////
vec3 EvaluateSH( vec3 vNormal )
{
    vec4 vNrm = vec4( vNormal.x, -vNormal.y, vNormal.z, 1.0 );

    // Evaluate SH
    vec3 x1, x2, x3;
    
    // Linear + constant polynomial terms
    x1.r = dot( g_vSHAr, vNrm );
    x1.g = dot( g_vSHAg, vNrm );
    x1.b = dot( g_vSHAb, vNrm );
    
    // 4 of the quadratic polynomials
    vec4 vB = vNrm.xyzz * vNrm.yzzx;   
    x2.r = dot( g_vSHBr, vB );
    x2.g = dot( g_vSHBg, vB );
    x2.b = dot( g_vSHBb, vB );
   
    // Final quadratic polynomial
    float vC = vNrm.x*vNrm.x - vNrm.y*vNrm.y;
    x3 = g_vSHC.rgb * vC;    

    // Sum terms and output
    return (x1 + x2 + x3) * g_fSHScaler;
}
////////////////////////////////////////////////////////////////////////////////////////////////////
void main(void)
{	
	vTexCoord = uv_attr;
	
	vec3 vN = normalize( mat3( g_mWorldTM[0].xyz, g_mWorldTM[1].xyz, g_mWorldTM[2].xyz ) * normal_attr );
	vColor = EvaluateSH(vN);
	vWorldNormal = vN;
	
	vec4 vWorldPos4 = g_mWorldTM * vec4(pos_attr.xyz, 1.0);
    vWorldPos = vWorldPos4.xyz;
    
    // Output vertex position
    gl_Position = g_mViewProjTM * vWorldPos4;
}