////////////////////////////////////////////////////////////////////////////////////////////////////
// HDR RGBE encoding post processing fragment shader
// 
////////////////////////////////////////////////////////////////////////////////////////////////////

// OpenGL ES require that precision is defined for a fragment shader
#ifndef NO_PRECISION_DEFINITION
	#ifdef GL_FRAGMENT_PRECISION_HIGH
	   precision highp float;
	#else
	   precision mediump float;
	#endif
#endif




////////////////////////////////////////////////////////////////////////////////////////////////////
uniform sampler2D g_texture;



////////////////////////////////////////////////////////////////////////////////////////////////////
varying vec2      uv_var;



////////////////////////////////////////////////////////////////////////////////////////////////////
vec4 EncodeRGBE8( in vec3 rgb )
{
	vec4 vEncoded;

    // Determine the largest color component
	float maxComponent = max( max( rgb.r, rgb.g ), rgb.b );
	
	// Round to the nearest integer exponent
	float fExp = ceil( log2( maxComponent ) );

    // Divide the components by the shared exponent
	vEncoded.rgb = rgb / exp2( fExp );
	
	// Store the shared exponent in the alpha channel
	vEncoded.a = ( fExp + 128.0 ) / 255.0;

	return vEncoded;
}



////////////////////////////////////////////////////////////////////////////////////////////////////
void main()
{
    //gl_FragColor = EncodeRGBE8( texture2D( g_texture, uv_var ).rgb * 10.0 ); // Alacran
    gl_FragColor = EncodeRGBE8( texture2D( g_texture, uv_var ).rgb );   // Oasis
}

