////////////////////////////////////////////////////////////////////////////////////////////////////
// Oasis - Ball vertex shader
// 
////////////////////////////////////////////////////////////////////////////////////////////////////



////////////////////////////////////////////////////////////////////////////////////////////////////
uniform mat4      g_mWorldTM;                   // PARAM_WORLD_TRANSFORM
uniform mat4      g_mViewProjTM;                // PARAM_VIEWPROJ_TRANSFORM



// SH data
uniform vec4 g_vSHAr;
uniform vec4 g_vSHAg;
uniform vec4 g_vSHAb;
uniform vec4 g_vSHBr;
uniform vec4 g_vSHBg;
uniform vec4 g_vSHBb;
uniform vec4 g_vSHC;
uniform float g_fSHScaler;



////////////////////////////////////////////////////////////////////////////////////////////////////
attribute vec3    pos_attr;                     // INDEX_POS_ATTR
attribute vec3    normal_attr;                  // INDEX_NORMAL_ATTR



////////////////////////////////////////////////////////////////////////////////////////////////////
varying vec4      vColor;



////////////////////////////////////////////////////////////////////////////////////////////////////
vec3 EvaluateSH( vec3 vNormal )
{
    vec4 vNrm = vec4( vNormal.x, -vNormal.y, vNormal.z, 1.0 );

    // Evaluate SH
    vec3 x1, x2, x3;
    
    // Linear + constant polynomial terms
    x1.r = dot( g_vSHAr, vNrm );
    x1.g = dot( g_vSHAg, vNrm );
    x1.b = dot( g_vSHAb, vNrm );
    
    // 4 of the quadratic polynomials
    vec4 vB = vNrm.xyzz * vNrm.yzzx;   
    x2.r = dot( g_vSHBr, vB );
    x2.g = dot( g_vSHBg, vB );
    x2.b = dot( g_vSHBb, vB );
   
    // Final quadratic polynomial
    float vC = vNrm.x*vNrm.x - vNrm.y*vNrm.y;
    x3 = g_vSHC.rgb * vC;    

    // Sum terms and output
    return (x1 + x2 + x3) * g_fSHScaler;
}



////////////////////////////////////////////////////////////////////////////////////////////////////
void main(void)
{
    // Transform position and normal
    vec4 vWorldPos = g_mWorldTM * vec4( pos_attr.xyz, 1.0 );
    vec3 vWorldNormal = normalize( mat3(g_mWorldTM[0].xyz, g_mWorldTM[1].xyz, g_mWorldTM[2].xyz) * normal_attr );
    vec4 vPos = g_mViewProjTM * vWorldPos;   

    // Calc radiance
    vColor.rgb = EvaluateSH( vWorldNormal );
    vColor.a = 1.0;

    // Output position
    gl_Position = vPos;
}


