////////////////////////////////////////////////////////////////////////////////////////////////////
// Experiment 13 - Skel fragment shader
////////////////////////////////////////////////////////////////////////////////////////////////////

// OpenGL ES require that precision is defined for a fragment shader
#ifdef GL_FRAGMENT_PRECISION_HIGH
   // Default precision
   precision highp float;
#else
   precision mediump float;
#endif

////////////////////////////////////////////////////////////////////////////////////////////////////
// UNIFORM
uniform sampler2D   g_texture;
uniform vec4	  g_vSHAr;
uniform vec4	  g_vSHAg;
uniform vec4	  g_vSHAb;
uniform vec4	  g_vSHBr;
uniform vec4	  g_vSHBg;
uniform vec4	  g_vSHBb;
uniform vec4	  g_vSHC;
uniform float	  g_fSHScaler;

////////////////////////////////////////////////////////////////////////////////////////////////////
// VARYING
varying vec2	  vTexCoord;
varying vec3	  vWorldNormal;

////////////////////////////////////////////////////////////////////////////////////////////////////

vec3 EvaluateSH( vec3 vNormal )
{
    vec4 vNrm = vec4( vNormal.x, -vNormal.y, vNormal.z, 1.0 );

    // Evaluate SH
    vec3 x1, x2, x3;
    
    // Linear + constant polynomial terms
    x1.r = dot( g_vSHAr, vNrm );
    x1.g = dot( g_vSHAg, vNrm );
    x1.b = dot( g_vSHAb, vNrm );
    
    // 4 of the quadratic polynomials
    vec4 vB = vNrm.xyzz * vNrm.yzzx;   
    x2.r = dot( g_vSHBr, vB );
    x2.g = dot( g_vSHBg, vB );
    x2.b = dot( g_vSHBb, vB );
   
    // Final quadratic polynomial
    float vC = vNrm.x*vNrm.x - vNrm.y*vNrm.y;
    x3 = g_vSHC.rgb * vC;    

    // Sum terms and output
    return (x1 + x2 + x3) * g_fSHScaler;
}

void main(void)
{
	vec4 vDiffuse = texture2D( g_texture, vTexCoord );
	vec3 vSH = EvaluateSH(vWorldNormal);
	
	gl_FragColor = vec4(vSH*vDiffuse.rgb,vDiffuse.a);
}
