////////////////////////////////////////////////////////////////////////////////////////////////////
// Experiment 13 - Skel vertex shader
////////////////////////////////////////////////////////////////////////////////////////////////////

////////////////////////////////////////////////////////////////////////////////////////////////////
// ATTRIBUTE
attribute vec3    pos_attr;
attribute vec3    normal_attr;
attribute vec2    uv_attr;
//attribute float   bone_attr;
attribute vec4    bone_attr2_weighted;
////////////////////////////////////////////////////////////////////////////////////////////////////
// UNIFORM
uniform mat4      g_mViewProjTM;
uniform vec4      g_vBoneTransforms[63*3];

////////////////////////////////////////////////////////////////////////////////////////////////////
// VARYING
varying vec2      vTexCoord;
varying vec3	  vWorldNormal;

////////////////////////////////////////////////////////////////////////////////////////////////////
mat4 skel_transform(float bone_idx)
{
    // Bone indices
    int bone = int(bone_idx);
    int datapoint1 = 3*bone+0;
    int datapoint2 = 3*bone+1;
    int datapoint3 = 3*bone+2;

    // Get our mat4 elements by swizzling around our bone transform data to correct elements
    vec4 v1 = vec4(g_vBoneTransforms[datapoint1].xyz, 0.0);
    vec4 v2 = vec4(g_vBoneTransforms[datapoint1].w, g_vBoneTransforms[datapoint2].xy, 0.0);
    vec4 v3 = vec4(g_vBoneTransforms[datapoint2].zw, g_vBoneTransforms[datapoint3].x, 0.0);
    vec4 v4 = vec4(g_vBoneTransforms[datapoint3].yzw, 1.0);
   
    return mat4(v1, v2, v3, v4);
}

////////////////////////////////////////////////////////////////////////////////////////////////////
vec4 skin_pos( vec4 vVec, mat4 boneTM1, mat4 boneTM2, vec2 vWeight )
{
    vec4 vSkinned = (boneTM1 * vVec) * vWeight.x;
    vSkinned += (boneTM2 * vVec) * vWeight.y;
    return vSkinned;
}

////////////////////////////////////////////////////////////////////////////////////////////////////
vec3 skin_vec( vec3 vVec, mat3 boneTM1, mat3 boneTM2, vec2 vWeight )
{
    vec3 vSkinned = (boneTM1 * vVec) * vWeight.x;
    vSkinned += (boneTM2 * vVec) * vWeight.y;
    return vSkinned;
}

////////////////////////////////////////////////////////////////////////////////////////////////////
void main(void)
{
	vTexCoord      = uv_attr;

	// Get bone transforms
    mat4 boneTM1 = skel_transform(bone_attr2_weighted.x);
    mat4 boneTM2 = skel_transform(bone_attr2_weighted.z);
    mat3 vecTM1 = mat3( boneTM1[0].xyz, boneTM1[1].xyz, boneTM1[2].xyz );
    mat3 vecTM2 = mat3( boneTM2[0].xyz, boneTM2[1].xyz, boneTM2[2].xyz );
    vec2 vWeight = vec2( bone_attr2_weighted.y, bone_attr2_weighted.w );
    
    // Transform position and normal
    vec4 vWorldPos = skin_pos( vec4( pos_attr.xyz, 1.0 ), boneTM1, boneTM2, vWeight );
    vWorldNormal = normalize(skin_vec( normal_attr, vecTM1, vecTM2, vWeight ));
    
	// Skel transforms
    //mat4 boneTM	= skel_transform(bone_attr);
    //mat3 mWorldBone = mat3(boneTM[0].xyz, boneTM[1].xyz, boneTM[2].xyz);
    
    //vec4 vWorldPos = boneTM * vec4(pos_attr.xyz, 1.0);
    //vWorldNormal = normalize(mWorldBone*normal_attr);
	
    // Output vertex position
    gl_Position = g_mViewProjTM * vWorldPos;
}