////////////////////////////////////////////////////////////////////////////////////////////////////
// Experiment 13 - Skel vertex shader
////////////////////////////////////////////////////////////////////////////////////////////////////

////////////////////////////////////////////////////////////////////////////////////////////////////
// ATTRIBUTE
attribute vec3    pos_attr;
attribute vec3    normal_attr;
attribute vec2    uv_attr;
attribute float   bone_attr;
////////////////////////////////////////////////////////////////////////////////////////////////////
// UNIFORM
uniform mat4      g_mViewProjTM;
uniform vec4      g_vBoneTransforms[63*3];

////////////////////////////////////////////////////////////////////////////////////////////////////
// VARYING
varying vec2      vTexCoord;
varying vec3	  vWorldNormal;

////////////////////////////////////////////////////////////////////////////////////////////////////
mat4 skel_transform(float bone_idx)
{
    // Bone indices
    int bone = int(bone_idx);
    int datapoint1 = 3*bone+0;
    int datapoint2 = 3*bone+1;
    int datapoint3 = 3*bone+2;

    // Get our mat4 elements by swizzling around our bone transform data to correct elements
    vec4 v1 = vec4(g_vBoneTransforms[datapoint1].xyz, 0.0);
    vec4 v2 = vec4(g_vBoneTransforms[datapoint1].w, g_vBoneTransforms[datapoint2].xy, 0.0);
    vec4 v3 = vec4(g_vBoneTransforms[datapoint2].zw, g_vBoneTransforms[datapoint3].x, 0.0);
    vec4 v4 = vec4(g_vBoneTransforms[datapoint3].yzw, 1.0);
   
    return mat4(v1, v2, v3, v4);
}

////////////////////////////////////////////////////////////////////////////////////////////////////
void main(void)
{
	vTexCoord      = uv_attr;

	// Skel transforms
    mat4 boneTM	= skel_transform(bone_attr);
    mat3 mWorldBone = mat3(boneTM[0].xyz, boneTM[1].xyz, boneTM[2].xyz);
    
    vec4 vWorldPos = boneTM * vec4(pos_attr.xyz, 1.0);
    vWorldNormal = normalize(mWorldBone*normal_attr);
	
    // Output vertex position
    gl_Position = g_mViewProjTM * vWorldPos;
}